@@ -5,6 +5,8 @@
 #ifndef _platform_h
 #define _platform_h
 
+#include <stdlib.h>
+
 #ifdef MINGW
 
 #include <windows.h>
@@ -14,7 +16,7 @@ extern "C" {
 #endif
 
 #define fdstdout (GetStdHandle(STD_OUTPUT_HANDLE))
-/*#define fdtype HANDLE   see zxid.h */
+/*#define fdtype HANDLE   see zx.h */
 #define BADFD (INVALID_HANDLE_VALUE)
 #define closefile(x) (CloseHandle(x)?0:-1)
 #define openfile_ro(path) zx_CreateFile((path), GENERIC_READ, FILE_SHARE_READ, 0 /*security*/, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0)
@@ -22,6 +24,10 @@ extern "C" {
 #define getegid() 0
 #define stat(X,Y) zx_stat(X,Y)
 
+#define LOCK_EX 2
+#define LOCK_UN 8
+#define O_SYNC 04010000
+
 #ifdef WIN32CL
 /* The directory handling is quite different on Windows. The following
  * posix wrappers are implemented in zxdirent.c */
@@ -49,7 +55,10 @@ typedef struct stack_st STACK;  /* MSVC seems to have some problem with openssl/
 #define getpid()  0
 #define geteuid() 0
 #define getegid() 0
-#define getcwd(b,n) "cwd on supported by Win32"
+#define chdir(path) SetCurrentDirectory(path)
+#define getcwd(b,n) "getcwd not supported on Win32"  /* *** consider GetCurrentDirectory() */
+unsigned int sleep(unsigned int secs);
+unsigned int alarm(unsigned int secs);
 
 HANDLE zx_CreateFile(LPCTSTR lpFileName, 
 		     DWORD dwDesiredAccess, DWORD dwShareMode, 
@@ -97,12 +106,12 @@ extern "C" {
 #endif
 
 #define fdstdout 1
-/*#define fdtype int   see zxid.h */
+/*#define fdtype int   see zx.h */
 #define BADFD (-1)
 #define closefile(x) close(x)
 #define openfile_ro(path) open((path),O_RDONLY)
 
-#ifndef _UNISTD_H
+#if !defined(_UNISTD_H) && !defined(_UNISTD_H_)
 #define _UNISTD_H 1  /* Prevent confusing double inclusion. */
 #define _UNISTD_H_ 1 /* MacOS: Prevent confusing double inclusion. */
 /* We do not want to include unistd.h because it does not exist on Win32.
@@ -110,30 +119,58 @@ extern "C" {
  * indirectly included first. In general we believe these Unix APIs are
  * so standard that we do not need system includes and can cover
  * the very few exceptions as ifdefs right in here. --Sampo */
-int symlink(const char *oldpath, const char *newpath);
-int link(const char* old, const char* new);
-int unlink(const char *pathname);
-int rmdir(const char *pathname);
-int getpid(void);
+int chdir(const char* path);
+int close(int);
+int dup(int);
+int dup2(int,int);
+int execl(const char *path, const char *arg, ...);
+int fcntl(int fd, int cmd, ...);         /* Preferred */
+int fork(void);
+char* getcwd(char* buf, size_t size);
 int geteuid(void);
 int getegid(void);
-int setuid(int);
-int setgid(int);
-char* getcwd(char* buf, int size);
-int fork(void);
-int execl(const char *path, const char *arg, ...);
-int dup(int);
+int getpid(void);
+int link(const char* old, const char* new);
+int lockf(int fd, int cmd, int len);     /* Depends on current seek pos: problem in append */
+int lseek(int fd, int offset, int whence);
 int pipe(int fd[2]);
 int read(int fd, void* buf, int count);
+int rmdir(const char *pathname);
+int setuid(int);
+int setgid(int);
+int symlink(const char *oldpath, const char *newpath);
+int unlink(const char *pathname);
 int write(int fd, void* buf, int count);
-int lseek(int fd, int offset, int whence);
-int close(int);
-int lockf(int fd, int cmd, int len);     /* Depends on current seek pos: problem in append */
-int fcntl(int fd, int cmd, ...);         /* Preferred */
+unsigned int sleep(unsigned int secs);
+unsigned int alarm(unsigned int secs);
 #define F_LOCK 1
 #define F_ULOCK 0
 #endif
 
+#ifdef _GNU_SOURCE
+#include <mcheck.h>
+#endif
+
+#if defined(MACOSX) || defined(FREEBSD)
+#include <sys/event.h>      /* for kqueue used by zxbusd */
+#define EPOLLHUP (0)        /* *** Need to find better constant */
+#define EPOLLERR (0)        /* *** Need to find better constant */
+#define EPOLLOUT (EVFILT_WRITE)
+#define EPOLLIN  (EVFILT_READ)
+#endif
+
+#ifdef LINUX
+#include <sys/epoll.h>      /* See man 4 epoll (Linux 2.6) */
+#endif
+#ifdef SUNOS
+#include <sys/devpoll.h>    /* See man -s 7d poll (Solaris 8) */
+#include <sys/poll.h>
+#define EPOLLHUP (POLLHUP)  /* 0x010 */
+#define EPOLLERR (POLLERR)  /* 0x008 */
+#define EPOLLOUT (POLLOUT)  /* 0x004 */
+#define EPOLLIN  (POLLIN)   /* 0x001 */
+#endif
+
 #ifdef __cplusplus
 } // extern "C"
 #endif
