@@ -61,10 +61,9 @@ Usage: zxid [options]   (when used as CGI, no options can be supplied)\n\
                    or Provider ID, aka well known location. The imported metadata\n\
                    is written to CoT cache directory.\n\
   -fileimport FILE Import metadata of others from file.\n\
-  -C CONFPATH      Path to (optional) config file, default /var/zxid/zxid.conf\n\
-  -O OPT=VAL       Override default or config file option. Only after -C, if any.\n\
+  -C CONFPATH      Path to (optional) config file, default " ZXID_CONF_PATH "\n\
+  -c OPT=VAL       Override default or config file option. Only after -C, if any.\n\
   -t SECONDS       Timeout. Default: 0=no timeout.\n\
-  -c CIPHER        Enable crypto on DTS interface using specified cipher. Use '?' for list.\n\
   -k FDNUMBER      File descriptor for reading symmetric key. Use 0 for stdin.\n\
   -egd PATH        Specify path of Entropy Gathering Daemon socket, default on\n\
                    Solaris: /tmp/entropy. On Linux /dev/urandom is used instead\n\
@@ -80,7 +79,7 @@ Usage: zxid [options]   (when used as CGI, no options can be supplied)\n\
   -h               This help message\n\
   --               End of options\n";
 
-int afr_buf_size = 0;
+int ak_buf_size = 0;
 int verbose = 1;
 int timeout = 0;
 int gcthreshold = 0;
@@ -96,7 +95,7 @@ char buf[32*1024];
 /* N.B. This options processing is a skeleton. In reality CGI scripts do not have
  * an opportunity to process any options. */
 
-/* Called by:  main x8, zxcall_main, zxcot_main, zxdecode_main */
+/* Called by:  main x8, zxbusd_main, zxbuslist_main, zxbustailf_main, zxcall_main, zxcot_main, zxdecode_main */
 void opt(int* argc, char*** argv, char*** env, zxid_conf* cf, zxid_cgi* cgi)
 {
   char* conf_path = 0;
@@ -119,7 +118,7 @@ void opt(int* argc, char*** argv, char*** env, zxid_conf* cf, zxid_cgi* cgi)
       conf_path = **argv;
       continue;
 
-    case 'O': if ((*argv)[0][2]) break;
+    case 'c': if ((*argv)[0][2]) break;
       ++(*argv); --(*argc);
       if (!(*argc)) break;
       if (conf_path != (char*)1) {
@@ -206,7 +205,7 @@ void opt(int* argc, char*** argv, char*** env, zxid_conf* cf, zxid_cgi* cgi)
       case 'i':
 	if (!strcmp((*argv)[0],"-license")) {
 	  extern char* license;
-	  fprintf(stderr, license);
+	  fprintf(stderr, "%s", license);
 	  exit(0);
 	}
 	break;
@@ -236,7 +235,7 @@ void opt(int* argc, char*** argv, char*** env, zxid_conf* cf, zxid_cgi* cgi)
     case 'r':
       switch ((*argv)[0][2]) {
       case 'f':
-	/*AFR_TS(LEAK, 0, "memory leaks enabled");*/
+	/*AK_TS(LEAK, 0, "memory leaks enabled");*/
 	ERR("*** WARNING: You have turned memory frees to memory leaks. We will (eventually) run out of memory. Using -rf is not recommended. %d\n", 0);
 	++leak_free;
 	continue;
@@ -266,7 +265,7 @@ void opt(int* argc, char*** argv, char*** env, zxid_conf* cf, zxid_cgi* cgi)
 	continue;
       case 'a': /* -ra */
 	if ((*argv)[0][3] == 0) {
-	  /*AFR_TS(ASSERT_NONFATAL, 0, "assert nonfatal enabled");*/
+	  /*AK_TS(ASSERT_NONFATAL, 0, "assert nonfatal enabled");*/
 #if 1
 	  ERR("*** WARNING: YOU HAVE TURNED ASSERTS OFF USING -ra FLAG. THIS MEANS THAT YOU WILL NOT BE ABLE TO OBTAIN ANY SUPPORT. IF PROGRAM NOW TRIES TO ASSERT IT MAY MYSTERIOUSLY AND UNPREDICTABLY CRASH INSTEAD, AND NOBODY WILL BE ABLE TO FIGURE OUT WHAT WENT WRONG OR HOW MUCH DAMAGE MAY BE DONE. USING -ra IS NOT RECOMMENDED. %d\n", assert_nonfatal);
 #endif
@@ -417,6 +416,7 @@ int main(int argc, char** argv, char** env)
   char* qs;
   char* cont_len;
   struct zx_str* ss;
+  char* eid;
   zxid_entity* idp;
   
 #if 1
@@ -427,6 +427,7 @@ int main(int argc, char** argv, char** env)
     exit(2);
   fprintf(stderr, "=================== Running ===================\n");
   ++zx_debug;
+  zxid_set_opt(cf, 6, 0);
 #endif
   cf->nosig_fatal = 0;  // *** For SimpleSign the signature is checked at other level
 
@@ -462,7 +463,7 @@ int main(int argc, char** argv, char** env)
   qs = getenv("QUERY_STRING");
   if (qs) {
     D("QS(%s)", qs);
-    zxid_parse_cgi(&cgi, qs);
+    zxid_parse_cgi(cf, &cgi, qs);
     if (cgi.op == 'P') {
       cont_len = getenv("CONTENT_LENGTH");
       if (cont_len) {
@@ -481,7 +482,7 @@ int main(int argc, char** argv, char** env)
 	  if (buf[3] == '<') {  /* UTF-8 BOM and looks XML */
 	    return zxid_sp_soap_parse(cf, &cgi, &ses, got-3, buf+3);
 	  }
-	  zxid_parse_cgi(&cgi, buf);
+	  zxid_parse_cgi(cf, &cgi, buf);
 	}
       }
     }
@@ -506,7 +507,9 @@ int main(int argc, char** argv, char** env)
     printf("Location: %s?o=C\r\n\r\n", ZXID_CDC_URL);
     return 0;
   case 'C':  /* CDC Read: Common Domain Cookie Reader */
-    return zxid_cdc_read(cf, &cgi);
+    if (zxid_cdc_read(cf, &cgi))
+      return 0;
+    return 1;
   case 'E':  /* Return from CDC read, or start here to by-pass CDC read. */
     if (zxid_lecp_check(cf, &cgi))
       return 0;    
@@ -592,9 +595,8 @@ int main(int argc, char** argv, char** env)
 #endif
 
   printf("<h3>CoT configuration parameters your IdP may need to know</h3>\n");
-  ss = zxid_my_ent_id(cf);
-  printf("Entity ID of this SP: <a href=\"%.*s\">%.*s</a> (Click on the link to fetch SP metadata.)\n",
-	 ss->len, ss->s, ss->len, ss->s);
+  eid = zxid_my_ent_id_cstr(cf);
+  printf("Entity ID of this SP: <a href=\"%s\">%s</a> (Click on the link to fetch SP metadata.)\n", eid, eid);
 
   printf("<h3>Technical options (typically hidden fields on production site)</h3>\n");
   printf("<input type=checkbox name=fc value=1 checked> Allow new federation to be created<br>\n");
