@@ -1,4 +1,5 @@
 /* zxid.h  -  Definitions for zxid CGI
+ * Copyright (c) 2012-2013 Synergetics NV (sampo@synergetics.be), All Rights Reserved.
  * Copyright (c) 2009-2011 Sampo Kellomaki (sampo@iki.fi), All Rights Reserved.
  * Copyright (c) 2006-2009 Symlabs (symlabs@symlabs.com), All Rights Reserved.
  * Author: Sampo Kellomaki (sampo@iki.fi)
@@ -19,6 +20,10 @@
  * 31.5.2010,  eliminated many include dependencies from the public API --Sampo
  * 13.11.2010, added ZXID_DECL for benefit of the Windows port --Sampo
  * 12.12.2010, separate zxidpriv.h and zxidutil.h from zxid.h --Sampo
+ * 17.8.2012,  added audit bus configuration --Sampo
+ * 16.2.2013,  added WD option --Sampo
+ * 14.3.2013   added language/skin dependent templates --Sampo
+ * 21.6.2013,  added wsp_pat --Sampo
  */
 
 #ifndef _zxid_h
@@ -30,6 +35,13 @@
 #ifdef USE_CURL
 #include <curl/curl.h>
 #endif
+#ifdef USE_OPENSSL
+#include <openssl/ssl.h>
+#endif
+
+#ifndef ZXID_CONF_PATH
+#define ZXID_CONF_PATH "/var/zxid/zxid.conf"
+#endif
 
 #include <zx/zx.h>
 
@@ -91,6 +103,7 @@ struct zxid_entity_s {
   struct zxid_entity_s* n_cdc;  /* *** not thread safe */
   char* eid;            /* Entity ID. Always nul terminated. */
   char* dpy_name;       /* OrganizationDisplayName. Always nul terminated. */
+  char* button_url;     /* OrganizationURL. Used for branding buttons in IdP sel screen, etc. */
   char  sha1_name[28];  /* 27 chars (+1 that is overwritten with nul) */
   struct zx_md_EntityDescriptor_s* ed;  /* Metadata */
   struct zxid_map* aamap;  /* Optional. Read from /var/zxid/idpuid/.all/sp_name_buf/.cf */
@@ -133,9 +146,9 @@ typedef struct zxid_ses     zxid_ses;
 
 struct zxid_conf {
   unsigned int magic;
-  struct zx_ctx* ctx;        /* ZX parsing context. Usually used for memory allocation. */
+  struct zx_ctx* ctx; /* ZX parsing context. Usually used for memory allocation. */
   zxid_entity* cot;   /* Linked list of metadata for CoT partners (in-memory CoT cache) */
-  int path_supplied;         /* FLAG: If config variable PATH is supplied, it may trigger reading config file from the supplied location. */
+  int path_supplied;  /* FLAG: If config variable PATH is supplied, it may trigger reading config file from the supplied location. */
   int path_len;
   char* path;
   char* url;
@@ -192,8 +205,10 @@ struct zxid_conf {
   
   char* affiliation;
   char* nice_name;           /* Human readable "nice" name. Used in AuthnReq->ProviderName */
+  char* button_url;          /* OrganizationURL. Used for branding buttons. */
+  char* pref_button_size;    /* Preferred branding button size (thers are ignored). */
   char* org_name;
-  char* org_url;
+  /*char* org_url;           renamed as button_url and given new semantics */
   char* locality;            /* Used for CSR locality (L) field. */
   char* state;               /* Used for CSR state (ST) field. */
   char* country;             /* Used for CSR country (C) field. */
@@ -204,6 +219,7 @@ struct zxid_conf {
   char* fedusername_suffix;  /* Default is computed from url domain name part when url is set. */
   char* ses_arch_dir;        /* Place where dead sessions go. 0=rm */
   char* ses_cookie_name;
+  char* ptm_cookie_name;
   char* ipport;              /* Source IP and port for logging, e.g: "1.2.3.4:5" */
     
   char* load_cot_cache;
@@ -222,6 +238,8 @@ struct zxid_conf {
   char* xasp_vers;
   char* trustpdp_url;
   char* defaultqs;
+  char* wsp_pat;
+  char* sso_pat;
   char* mod_saml_attr_prefix;  /* Prefix for req variables in mod_auth_saml */
 
   struct zxid_need*  need;
@@ -242,9 +260,9 @@ struct zxid_conf {
   struct zxid_cstr_list* localpdp_idpnid_deny;
 
   char* wsc_localpdp_obl_pledge;
-  char* wsp_localpdp_obl_req;
+  struct zxid_obl_list* wsp_localpdp_obl_req;
   char* wsp_localpdp_obl_emit;
-  char* wsc_localpdp_obl_accept;
+  struct zxid_obl_list* wsc_localpdp_obl_accept;
   
   int   bootstrap_level;     /* How many layers of bootstraps are generated. */
   int   max_soap_retry;      /* How many times a ID-WSF SOAP call can be retried (update EPR) */
@@ -282,6 +300,10 @@ struct zxid_conf {
   char* mgmt_end;      /* End of page, after version string */
 
   char* dbg;           /* Debug message that may be shown. */
+  char* wd;            /* Forced working directory. */
+
+  struct zxid_bus_url* bus_url;  /* Audit bus URLs to contact. */
+  char*  bus_pw;             /* Audit bus password if not using ClientTLS */
 
   char  log_err;             /* Log enables and signing and encryption flags (if USE_OPENSSL) */
   char  log_act;
@@ -306,14 +328,16 @@ struct zxid_conf {
   char  az_opt;        /* Kludgy options for AZ debugging and to work-around bugs of others */
   char  valid_opt;     /* Kludgy options for AZ debugging and to work-around bugs of others */
   char  idp_pxy_ena;
-  char  pad2; char pad3; char pad4; char pad5; char pad6; char pad7;
+  char  oaz_jwt_sigenc_alg;  /* What signature and encryption to apply to issued JWT (OAUTH2) */
+  char  bus_rcpt;            /* Audit Bus receipt enable and signing flags */
+  char  pad5; char pad6; char pad7;
 
 #ifdef USE_CURL
   CURL* curl;
 #endif
 #ifdef USE_PTHREAD
-  pthread_mutex_t mx;
-  pthread_mutex_t curl_mx;   /* Avoid holding the main lock for duration of HTTP request */
+  struct zx_lock mx;
+  struct zx_lock curl_mx;   /* Avoid holding the main lock for duration of HTTP request */
 #endif
 #ifdef USE_OPENSSL
   EVP_PKEY*  sign_pkey;
@@ -323,8 +347,10 @@ struct zxid_conf {
 
   char  psobj_symkey[20];    /* sha1 hash of key data */
   char  log_symkey[20];      /* sha1 hash of key data */
+  char  hmac_key[20];        /* sha1 hash of key data */
   EVP_PKEY*  log_sign_pkey;
   X509* log_enc_cert;
+  SSL_CTX* ssl_ctx;
 #endif
 };
 
@@ -359,7 +385,7 @@ struct zxid_cgi {
   char* pxy_count;     /* ProxyCount for triggering IdP proxying */
   char* get_complete;  /* GetComplete URL for IdP proxying */
   char* idppxylist;    /* IDPList for IdP proxying */
-  char* rs;            /* RelayState in redirect profile. mod_auth_saml: uri to access after SSO */
+  char* rs;            /* RelayState in redirect profile. mod_auth_saml, SSO servlet: def-sb64 armored uri to access after SSO */
   char* newnym;        /* New NameID for MNI/nireg. Empty for federation termination. */
   char* saml_art;      /* SAMLart=... artifact, as in artifact consumer URL. */
   char* saml_resp;     /* SAMLResponse=... in redirect profile */
@@ -374,9 +400,50 @@ struct zxid_cgi {
   char* zxapp;         /* Deployment specific application parameter passed in some querystrings. */
   char* zxrfr;         /* ZX Referer. Indicates to some external pages why user was redirected. */
   char* ok;            /* Ok button in some forms */
+  char* templ;         /* Template name in some forms (used to implement tabs, e.g. in idpsel) */
   char* sp_eid;        /* IdP An for to generate page */
   char* sp_dpy_name;
+  char* sp_button_url;
+  char* response_type; /* OAuth2 / OpenID-Connect */
+  char* client_id;     /* OAuth2 */
+  char* scope;         /* OAuth2 */
+  char* redirect_uri;  /* OAuth2 */
+  char* nonce;         /* OAuth2 */
+  char* state;         /* OAuth2 */
+  char* display;       /* OAuth2 */
+  char* prompt;        /* OAuth2 */
+  char* access_token;  /* OAuth2 */
+  char* token_type;    /* OAuth2 */
+  char* id_token;      /* OAuth2 */
+  char* expires_in;    /* OAuth2 */
+  char* iss;           /* OAuth2 */
+  char* user_id;       /* OAuth2 */
+  char* aud;           /* OAuth2 */
+  char* exp;           /* OAuth2 */
+  char* iso29115;      /* OAuth2 */
+  char* schema;        /* OAuth2 */
+  char* id;            /* OAuth2 */
+#if 0
+  char* name;          /* OAuth2 */
+  char* given_name;    /* OAuth2 */
+  char* family_name;   /* OAuth2 */
+  char* middle_name;   /* OAuth2 */
+  char* nickname;      /* OAuth2 */
+  char* profile;       /* OAuth2 */
+  char* picture;       /* OAuth2 */
+  char* website;       /* OAuth2 */
+  char* email;         /* OAuth2 */
+  char* verified;      /* OAuth2 */
+  char* gender;        /* OAuth2 */
+  char* birthday;      /* OAuth2 */
+  char* zoneinfo;      /* OAuth2 */
+  char* locale;        /* OAuth2 */
+  char* phone_number;  /* OAuth2 */
+  char* address;       /* OAuth2 */
+  char* updated_time;  /* OAuth2 */
+#endif
   char* inv;           /* Invitation ID */
+  char* skin;
   char* action_url;    /* action URL in some forms, such as post.html */
   zxid_entity* idp_list;   /* IdPs from CDC */
 };
@@ -385,7 +452,7 @@ struct zxid_cgi {
 
 struct zxid_ses {
   unsigned int magic;
-  char* sid;
+  char* sid;           /* Session ID. Same as in cookie, same as file name */
   char* uid;           /* Local uid (only if local login, like in IdP) */
   char* nid;           /* String representation of Subject NameID. See also nameid. */
   char* tgt;           /* String representation of Target NameID. See also nameid. */
@@ -401,6 +468,7 @@ struct zxid_ses {
   char* sso_a7n_path;  /* Reference to the SSO assertion (needed for SLO) */
   char* tgt_a7n_path;  /* Reference to target identity assertion */
   char* setcookie;     /* If set, the content rendering should include set-cookie header. */
+  char* setptmcookie;  /* For PTM related set-cookie header. */
   char* cookie;        /* Cookie seen by downstream internal requests after SSO. */
   char* rs;            /* RelayState at SSO. mod_auth_saml uses this as URI after SSO. */
   char* rcvd_usagedir; /* Received Usage Directives. Populated by zxid_wsc_validate_resp_env() */
@@ -424,7 +492,7 @@ struct zxid_ses {
   char* sso_a7n_buf;
   struct zxid_attr* at; /* Attributes extracted from a7n and translated using inmap. Linked list */
 #ifdef USE_PTHREAD
-  pthread_mutex_t mx;
+  struct zx_lock mx;
 #endif
 };
 
@@ -462,13 +530,21 @@ struct zxid_map {
   char* ext;
 };
 
-/*(s) Used for maintaining whitelists and blacklists */
+/*(s) Used for maintaining whitelists and blacklists as well as obligation values */
 
 struct zxid_cstr_list {
   struct zxid_cstr_list* n;
   char* s;
 };
 
+/*(s) Obligations list with multiple values per obligation. */
+
+struct zxid_obl_list {
+  struct zxid_obl_list* n;
+  char* name;
+  struct zxid_cstr_list* vals;
+};
+
 #define ZXID_MAP_RULE_RENAME     0x00
 #define ZXID_MAP_RULE_DEL        0x01  /* Filter attribute out */
 #define ZXID_MAP_RULE_RESET      0x02  /* Reset the map, dropping previous config. */
@@ -484,6 +560,36 @@ struct zxid_cstr_list {
 #define ZXID_MAP_RULE_WRAP_FILE  0x30  /* Get attribute value from file specified in ext */
 #define ZXID_MAP_RULE_WRAP_MASK  0x30
 
+/*(s) Parsed STOMP 1.1 headers */
+
+struct stomp_hdr {
+  int len;              /* Populated from content-length header, if one is supplied. */
+  char* body;           /* Body of the message */
+  char* host;           /* also receipt and receipt_id */
+  char* vers;           /* version, also accept-version, tx_id */
+  char* login;          /* also session, subs_id, subsc */
+  char* pw;             /* also server, ack, msg_id */
+  char* dest;           /* destination, also heart_bt */
+  char* end_of_pdu;     /* One past end of frame data. Helps in cleaning buffer for next PDU. */
+};
+
+/*(s) Used for maintaining audit bus URL and connections */
+
+struct zxid_bus_url {
+  struct zxid_bus_url* n;
+  char* s;              /* The config URL */
+  char* eid;            /* EntityID of the auditbus node (for metadata and zx-rcpt-sig validate) */
+  fdtype fd;            /* Remember already open connection to zxbusd instance. */
+  char* m;              /* I/O buffer */
+  char* ap;             /* How far the buffer is filled */
+  int   cur_rcpt;       /* Rolling receipt ID */
+  char  scalingpart;    /* Scaling partition number. */
+  char  pad1,pad2,pad3;
+#ifdef USE_OPENSSL
+  SSL*  ssl;
+#endif
+};
+
 /*(s) Attribute source definition */
 
 struct zxid_atsrc {
@@ -548,6 +654,7 @@ struct zxid_invite {
 #define ZXID_COT_DIR  "cot/"
 #define ZXID_DIMD_DIR "dimd/"
 #define ZXID_INV_DIR  "inv/"
+#define ZXID_LOG_DIR  "log/"
 #define ZXID_MAX_USER (256)  /* Maximum size of .mni or user file */
 #define ZXID_INIT_MD_BUF   (8*1024-1)  /* Initial size, will automatically reallocate. */
 #define ZXID_INIT_SOAP_BUF (8*1024-1)  /* Initial size, will automatically reallocate. */
@@ -646,9 +753,11 @@ ZXID_DECL struct zx_str* zxenc_symkey_dec(zxid_conf* cf, struct zx_xenc_Encrypte
 /* zxlog (see logging chapter in README.zxid) */
 
 /*  /var/zxid/log/rely/ISSUER-SHA1-NAME/a7n/A7N-ID-AS-SHA1 */
+#define ZXBUS_CH_DIR    "ch/"
 #define ZXLOG_RELY_DIR  "rely/"
 #define ZXLOG_ISSUE_DIR "issue/"
 #define ZXLOG_A7N_KIND  "/a7n/"
+#define ZXLOG_JWT_KIND  "/jwt/"
 #define ZXLOG_MSG_KIND  "/msg/"
 #define ZXLOG_WIR_KIND  "/wir/"
 
@@ -659,6 +768,21 @@ ZXID_DECL int zxlog(zxid_conf* cf, struct timeval* ourts, struct timeval* srcts,
 ZXID_DECL int zxlogwsp(zxid_conf* cf, zxid_ses* ses, const char* res, const char* op, const char* arg, const char* fmt, ...);
 ZXID_DECL int zxlogusr(zxid_conf* cf, const char* uid, struct timeval* ourts, struct timeval* srcts, const char* ipport, struct zx_str* entid, struct zx_str* msgid, struct zx_str* a7nid, struct zx_str* nid, const char* sigval, const char* res, const char* op, const char* arg, const char* fmt, ...);
 ZXID_DECL void zxlog_debug_xml_blob(zxid_conf* cf, const char* file, int line, const char* func, const char* lk, int len, const char* xml);
+ZXID_DECL char* zxbus_mint_receipt(zxid_conf* cf, int sigbuf_len, char* sigbuf, int mid_len, const char* mid, int dest_len, const char* dest, int eid_len, const char* eid, int body_len, const char* body);
+ZXID_DECL int zxbus_verify_receipt(zxid_conf* cf, const char* eid, int sigbuf_len, char* sigbuf, int mid_len, const char* mid, int dest_len, const char* dest, int deid_len, const char* deid, int body_len, const char* body);
+ZXID_DECL int zxbus_persist_msg(zxid_conf* cf, int c_path_len, char* c_path, int dest_len, const char* dest, int data_len, const char* data);
+
+/* zxbusprod */
+
+ZXID_DECL int zxbus_open_bus_url(zxid_conf* cf, struct zxid_bus_url* bu);
+ZXID_DECL int zxbus_close(zxid_conf* cf, struct zxid_bus_url* bu);
+ZXID_DECL void zxbus_close_all(zxid_conf* cf);
+ZXID_DECL int zxbus_send_cmdf(zxid_conf* cf, struct zxid_bus_url* bu, int body_len, const char* body, const char* fmt, ...);
+ZXID_DECL int zxbus_send_cmd(zxid_conf* cf, const char* cmd, const char* dest, int body_len, const char* body);
+ZXID_DECL int zxbus_send(zxid_conf* cf, const char* dest, int body_len, const char* body);
+ZXID_DECL int zxbus_read_stomp(zxid_conf* cf, struct zxid_bus_url* bu, struct stomp_hdr* stomp);
+ZXID_DECL int zxbus_ack_msg(zxid_conf* cf, struct zxid_bus_url* bu, struct stomp_hdr* stompp);
+ZXID_DECL char* zxbus_listen_msg(zxid_conf* cf, struct zxid_bus_url* bu);
 
 /* zxidmeta */
 
@@ -669,7 +793,7 @@ ZXID_DECL zxid_entity* zxid_parse_meta(zxid_conf* cf, char** md, char* lim);
 ZXID_DECL zxid_entity* zxid_get_meta_ss(zxid_conf* cf, struct zx_str* url);
 ZXID_DECL zxid_entity* zxid_get_meta(zxid_conf* cf, const char* url);
 ZXID_DECL zxid_entity* zxid_get_ent_ss(zxid_conf* cf, struct zx_str* eid);
-ZXID_DECL zxid_entity* zxid_get_ent(zxid_conf* cf, char* eid);
+ZXID_DECL zxid_entity* zxid_get_ent(zxid_conf* cf, const char* eid);
 ZXID_DECL zxid_entity* zxid_get_ent_by_succinct_id(zxid_conf* cf, char* raw_succinct_id);
 ZXID_DECL zxid_entity* zxid_get_ent_by_sha1_name(zxid_conf* cf, char* sha1_name);
 ZXID_DECL zxid_entity* zxid_load_cot_cache(zxid_conf* cf);
@@ -678,6 +802,7 @@ ZXID_DECL struct zx_str* zxid_sp_meta(zxid_conf* cf, zxid_cgi* cgi);
 ZXID_DECL int zxid_send_sp_meta(zxid_conf* cf, zxid_cgi* cgi);
 ZXID_DECL struct zx_str* zxid_sp_carml(zxid_conf* cf);
 ZXID_DECL struct zx_str* zxid_my_ent_id(zxid_conf* cf);
+ZXID_DECL char* zxid_my_ent_id_cstr(zxid_conf* cf);
 ZXID_DECL struct zx_attr_s* zxid_my_ent_id_attr(zxid_conf* cf, struct zx_elem_s* father, int tok);
 ZXID_DECL struct zx_str* zxid_my_cdc_url(zxid_conf* cf);
 ZXID_DECL struct zx_sa_Issuer_s* zxid_my_issuer(zxid_conf* cf, struct zx_elem_s* father);
@@ -696,6 +821,7 @@ ZXID_DECL int   zxid_set_opt(zxid_conf* cf, int which, int val);
 ZXID_DECL char* zxid_set_opt_cstr(zxid_conf* cf, int which, char* val);
 ZXID_DECL void  zxid_url_set(zxid_conf* cf, const char* url);
 ZXID_DECL int   zxid_init_conf(zxid_conf* cf, const char* conf_dir);
+ZXID_DECL void zxid_free_conf(zxid_conf *cf);
 ZXID_DECL zxid_conf* zxid_init_conf_ctx(zxid_conf* cf, const char* zxid_path);
 ZXID_DECL zxid_conf* zxid_new_conf(const char* zxid_path);
 ZXID_DECL int   zxid_parse_conf_raw(zxid_conf* cf, int qs_len, char* qs);
@@ -706,7 +832,7 @@ ZXID_DECL struct zx_str* zxid_show_conf(zxid_conf* cf);
 
 /* zxidcgi */
 
-ZXID_DECL int zxid_parse_cgi(zxid_cgi* cgi, char* qs);
+ZXID_DECL int zxid_parse_cgi(zxid_conf* cf, zxid_cgi* cgi, char* qs);
 ZXID_DECL zxid_cgi* zxid_new_cgi(zxid_conf* cf, char* qs);
 ZXID_DECL void zxid_get_sid_from_cookie(zxid_conf* cf, zxid_cgi* cgi, const char* cookie);
 
@@ -728,7 +854,7 @@ ZXID_DECL struct zx_str* zxid_ses_to_json(zxid_conf* cf, zxid_ses* ses);
 ZXID_DECL struct zx_str* zxid_ses_to_qs(zxid_conf* cf, zxid_ses* ses);
 ZXID_DECL void zxid_ses_to_pool(zxid_conf* cf, zxid_ses* ses);
 ZXID_DECL void zxid_add_attr_to_ses(zxid_conf* cf, zxid_ses* ses, char* at_name, struct zx_str* val);
-ZXID_DECL int zxid_add_qs_to_ses(zxid_conf* cf, zxid_ses* ses, char* qs, int apply_map);
+ZXID_DECL int zxid_add_qs2ses(zxid_conf* cf, zxid_ses* ses, char* qs, int apply_map);
 
 /* zxiduser */
 
@@ -740,6 +866,7 @@ ZXID_DECL int zxid_pw_authn(zxid_conf* cf, zxid_cgi* cgi, zxid_ses* ses);
 
 /* zxidcurl */
 
+ZXID_DECL char* zxid_http_get(zxid_conf* cf, const char* url, char** lim);
 ZXID_DECL struct zx_str* zxid_http_post_raw(zxid_conf* cf, int url_len, const char* url, int len, const char* data);
 ZXID_DECL struct zx_root_s* zxid_soap_call_raw(zxid_conf* cf, struct zx_str* url, struct zx_e_Envelope_s* env, char** ret_enve);
 ZXID_DECL struct zx_root_s* zxid_soap_call_hdr_body(zxid_conf* cf, struct zx_str* url, struct zx_e_Header_s* hdr, struct zx_e_Body_s* body);
@@ -825,6 +952,10 @@ ZXID_DECL struct zx_sa_Attribute_s* zxid_find_attribute(zxid_a7n* a7n, int nfmt_
 ZXID_DECL struct zx_sp_Status_s* zxid_mk_Status(zxid_conf* cf, struct zx_elem_s* father, const char* sc1, const char* sc2, const char* msg);
 ZXID_DECL struct zx_sp_Status_s* zxid_OK(zxid_conf* cf, struct zx_elem_s* father);
 
+/* zxidoauth */
+
+ZXID_DECL struct zx_str* zxid_mk_oauth_az_req(zxid_conf* cf, zxid_cgi* cgi, struct zx_str* loc, char* relay_state);
+
 /* zxidmkwsf */
 
 ZXID_DECL struct zx_lu_Status_s* zxid_mk_lu_Status(zxid_conf* cf, struct zx_elem_s* father, const char* sc1, const char* sc2, const char* msg, const char* ref);
@@ -971,6 +1102,9 @@ ZXID_DECL char* zxid_ps_finalize_invite(zxid_conf* cf, zxid_cgi* cgi, zxid_ses*
 #define ZXID_SAML2_REDIR 6   /* for function of same name, see */
 #define ZXID_SAML2_URI   7
 
+/* Following are experimental protocol bindings (2011) */
+#define ZXID_OPID_CONNECT 8
+
 #define ZXID_SLO_SVC 1
 #define ZXID_MNI_SVC 2
 #define ZXID_ACS_SVC 3
