@@ -1,4 +1,5 @@
 /* zxidcgi.c  -  Handwritten functions for parsing SP specific CGI options
+ * Copyright (c) 2012-2013 Synergetics NV (sampo@synergetics.be), All Rights Reserved.
  * Copyright (c) 2010-2011 Sampo Kellomaki (sampo@iki.fi), All Rights Reserved.
  * Copyright (c) 2006-2009 Symlabs (symlabs@symlabs.com), All Rights Reserved.
  * Author: Sampo Kellomaki (sampo@iki.fi)
@@ -8,11 +9,13 @@
  * Licensed under Apache License 2.0, see file COPYING.
  * $Id: zxidcgi.c,v 1.33 2010-01-08 02:10:09 sampo Exp $
  *
- * 12.8.2006, created --Sampo
- * 16.1.2007, split from zxidlib.c --Sampo
+ * 12.8.2006,  created --Sampo
+ * 16.1.2007,  split from zxidlib.c --Sampo
  * 12.10.2007, added cookie scanning --Sampo
- * 7.10.2008, added documentation --Sampo
- *
+ * 7.10.2008,  added documentation --Sampo
+ * 10.12.2011, added OAuth2, OpenID Connect, and UMA support --Sampo
+ * 20.10.2012, made the fr to rs copy cause deflate safe base64 encode --Sampo
+ * 14.3.2013   added language/skin dependent templates --Sampo
  * See also: http://hoohoo.ncsa.uiuc.edu/cgi/interface.html (CGI specification)
  */
 
@@ -22,76 +25,80 @@
 #include "errmac.h"
 #include "zxid.h"
 #include "zxidconf.h"
+#include "zxidutil.h"
 
 /* ============== CGI Parsing ============== */
 
 /*(i) Parse query string or form POST and detect parameters relevant for ZXID.
  * N.B. This CGI parsing is very specific for needs of ZXID. It is not generic.
  *
+ * cf:: ZXID configuration object, for occasional memory allocation.
  * cgi:: Already allocated CGI structure where results of this function
  *     are deposited. Note that this structure is not cleared. Thus it is
  *     possible to call zxid_parse_cgi() multiple times to accumulate
  *     results from multiple sources, e.g. foirst for query string, and then
  *     for form POST.
  * qs:: CGI formatted input. Usually query string or form POST content.
+ *     qs is modified in-place  e.g. to perform url decoding and nul termination.
+ *     References to the qs will be taken, e.g. pointers in cgi struct point
+ *     to it, thus qs MUST NOT be freed before cgi struct is freed, which
+ *     may be long after the call to zxid_parse_cgi().
  * return:: 0 on success. Other values reserved. Usually return value is
  *     ignored as there really is no way for this function to fail. Unrecognized
  *     CGI arguments are simply ignored with assumption that some other processing
  *     layer will pick them up - hence no need to flag error. */
 
 /* Called by:  chkuid x3, main x4, zxid_decode_ssoreq, zxid_new_cgi, zxid_simple_cf_ses x3 */
-int zxid_parse_cgi(zxid_cgi* cgi, char* qs)
+int zxid_parse_cgi(zxid_conf* cf, zxid_cgi* cgi, char* qs)
 {
-  char *p, *n, *v, *val, *name;
-  DD("qs(%s) len=%d", STRNULLCHK(qs), qs?strlen(qs):-1);
+  char *p, *n, *v;
+  DD("qs(%s)=%p len=%d", STRNULLCHK(qs), qs, qs?strlen(qs):-1);
   if (!qs)
     return 0;
   while (qs && *qs) {
-    for (; *qs == '&'; ++qs) ;                  /* Skip over & or && */
-    if (!*qs) break;
-    
-    qs = strchr(name = qs, '=');                /* Scan name (until '=') */
-    if (!qs) break;
-    if (qs == name) {                           /* Key was an empty string: skip it */
-      qs = strchr(qs, '&');                     /* Scan value (until '&') *** or '?' */
-      continue;
-    }
-    for (; name < qs && *name <= ' '; ++name) ; /* Skip over initial whitespace before name */
-    n = p = name;
-    URL_DECODE(p, name, qs);
-    *p = 0;                                     /* Nul-term n (name) */
-    
-    for (val = ++qs; *qs && *qs != '&'; ++qs) ; /* Skip over = and scan value (until '&') */
-    v = p = val;
-    /* SAMLRequest and Response MUST NOT be URL decoded as the URL encoding
-     * is needed for redirect binding signature validation. See also unbase64_raw()
-     * for how these fields are URL decoded at later stage. */
-    if (n[0] != 'S' && n[0] != 'R'
-	|| strcmp(n, "SAMLRequest") && strcmp(n, "SAMLResponse")
-	&& strcmp(n, "SigAlg") && strcmp(n, "Signature") && strcmp(n, "RelayState"))
-      URL_DECODE(p, val, qs);
-    else
-      p = qs;
-
-    if (*qs)
-      ++qs;
-    *p = 0;                                     /* Nul-term v (value) */
-    
+    qs = zxid_qs_nv_scan(qs, &n, &v, 2);
+    if (!n)
+      n = "NULL_NAME_ERROR";
+    DD("n(%s)=v(%s) qs(%s)=%p len=%d", n,v, STRNULLCHKNULL(qs), qs, qs?strlen(qs):-1);
     switch (n[0]) {
+    case 'n':
+      if (!strcmp(n, "nonce")) { cgi->nonce = v; break; }
+      goto unknown;
     case 'o':
       if (!n[1]) { cgi->op = v[0];    break; }
       if (n[1] = 'k' && !n[2]) { cgi->ok = v;  break; }  /* ok button */
       goto unknown;
+    case 'p':
+      if (!strcmp(n, "prompt")) { cgi->prompt = v; break; }
+      goto unknown;
+    case 'r':
+      if (!strcmp(n, "response_type")) { cgi->response_type = v; break; }
+      if (!strcmp(n, "redirect_uri"))  { cgi->redirect_uri = v;	 break; }
+      if (!strcmp(n, "rs"))            { cgi->rs = v; break; }
+      goto unknown;
     case 's':
       if (!n[1]) { cgi->sid = v; break; }
+      if (!strcmp(n, "scope"))  { cgi->scope = v; break; }
+      if (!strcmp(n, "state"))  { cgi->state = v; break; }
+      if (!strcmp(n, "schema")) { cgi->schema = v; break; }
+      goto unknown;
+    case 't':
+      if (!strcmp(n, "token_type")) { cgi->token_type = v; break; }
+      if (!strcmp(n, "templ"))   { DD("Detect templ(%s) cgi=%p",v,cgi); cgi->templ = v; break; }
+      goto unknown;
+    case 'u':
+      if (!strcmp(n, "user_id")) { cgi->user_id = v; break; }
       goto unknown;
     case 'c':
       if (!n[1]) { cgi->cdc = v; break; }
+      if (!strcmp(n, "client_id")) { cgi->client_id = v; break; }
       goto unknown;
       /* The following two entity IDs, combined with various login buttons
        * aim at supporting may different user interface layouts. You need to
        * understand how they interact to avoid undesired conflicts. */
     case 'e':  /* EntityID field (manual entry). Overrides 'd'. */
+      if (!strcmp(n, "expires_in")) { cgi->nonce = v; break; }
+      if (!strcmp(n, "exp"))        { cgi->exp = v; break; }
       if (!n[1]) {
 set_eid:
 	if (v[0]) cgi->eid = v;
@@ -107,6 +114,7 @@ set_eid:
 	}
 	goto set_eid;
       }
+      if (!strcmp(n, "display")) { cgi->display = v; break; }
       goto unknown;
     case 'l':
       /* Login button names are like lP<eid>, where "l" is literal ell, P is
@@ -114,16 +122,30 @@ set_eid:
        * N.B. If eid is omitted from button name, it may be provided using
        * d or e fields (see above). */
       cgi->pr_ix = n[1];
-      if (n[2])
+      if (n[2]) {
 	cgi->eid = n+2;
-      cgi->op = 'L';
-      D("cgi: login eid(%s)", cgi->eid);
-      break;
-    case 'i':
-      if (!strcmp(n, "inv")) {
-	cgi->inv = v;
-	break;
+	/*if (cf->idp_list_meth == ZXID_IDP_LIST_BRAND)*/
+	/* We need to remove the .x and/or .y from the end (image map coordinates) */
+	p = strchr(cgi->eid, 0);  /* Pointer to end of string */
+	DD("p[-2]=%x (%c%c) n=%p p=%p name(%s) val(%s)", p[-2], p[-2], p[-1], n, p, n, v);
+	if (p[-2] == '.' && ONE_OF_2(p[-1], 'x', 'y')) {
+	  p[-2] = 0;
+	  DD("eid=%p eid(%s) p[-2]=%x n=%p p=%p v=%p (%s)=(%s)", cgi->eid, cgi->eid, p[-2], n, p, v, n, v);
+	}
       }
+      cgi->op = 'L';
+      D("cgi: login eid=%p eid(%s)", cgi->eid, cgi->eid);
+      break;
+    case 'k':
+      DD("k CGI field(%s) val(%s) cgi=%p", n, v, cgi);
+      if (!n[1]) { cgi->skin = v;    break; }
+      goto unknown;
+    case 'i':
+      if (!strcmp(n, "id_token")) { cgi->id_token = v; break; }
+      if (!strcmp(n, "iss")) { cgi->iss = v; break; }
+      if (!strcmp(n, "iso29115")) { cgi->iso29115 = v; break; }
+      if (!strcmp(n, "id"))  { cgi->id = v; break; }
+      if (!strcmp(n, "inv")) { cgi->inv = v; break; }
       /* IdP and protocol index selection popup values are like P<eid>
        * N.B. If eid is omitted from button name, it may be provided using
        * d or e fields (see above). This effectively allows i to be just
@@ -145,7 +167,7 @@ set_eid:
       case 'n': cgi->nid_fmt = v;         D("nid_fmt=%s", cgi->nid_fmt); break;
       case 'p': cgi->ispassive = v[0];    D("ispassive=%c", cgi->ispassive); break;
       case 'q': cgi->affil = v;           break;
-      case 'r': cgi->rs = v;              break;
+      case 'r': cgi->rs = zxid_deflate_safe_b64_raw(cf->ctx, -2, v); break;
       case 'y': cgi->consent = v;         break;
       }
       break;
@@ -163,6 +185,8 @@ set_eid:
       break;
     case 'a':
       if (!n[1]) goto unknown;
+      if (!strcmp(n, "access_token")) { cgi->access_token = v; break; }
+      if (!strcmp(n, "aud"))          { cgi->aud = v; break; }
       switch (n[1]) {
       case 'l': if (n[3]) goto unknown;  cgi->op = n[2];           break;
       case 'u': if (n[2]) goto unknown;  if (v[0] || !cgi->uid) cgi->uid = v; break;
@@ -184,11 +208,8 @@ set_eid:
       }
       break;
     case 'R':
-      if (!strcmp(n, "RelayState")) {
-	cgi->rs = v;
-	break;
-      }
-      break;
+      if (!strcmp(n, "RelayState")) { cgi->rs = v; break; }
+      goto unknown;
     case 'S':
       if (!strcmp(n, "SAMLart")) {
 	cgi->saml_art = v;
@@ -216,7 +237,7 @@ set_eid:
       }
       /* fall thru */
     unknown:
-    default:  D("Unknown CGI field(%s) val(%s)", n, v);
+    default:  D("Unknown CGI field(%s) val(%s) cgi=%p", n, v, cgi);
     }
   }
   DD("END cgi=%p cgi->eid=%p (%s) op(%c) magic=%x", cgi, cgi->eid, cgi->eid, cgi->op, cgi->magic);
@@ -234,7 +255,7 @@ zxid_cgi* zxid_new_cgi(zxid_conf* cf, char* qs)
     qqs = ZX_ALLOC(cf->ctx, len+1);
     memcpy(qqs, qs, len);
     qqs[len] = 0;
-    zxid_parse_cgi(cgi, qqs);
+    zxid_parse_cgi(cf, cgi, qqs);
   }
   return cgi;
 }
