@@ -1,5 +1,6 @@
 /* zxididp.c  -  CGI binary for SAML 2 IdP
- * Copyright (c) 2008-2010 Sampo Kellomaki (sampo@iki.fi), All Rights Reserved.
+ * Copyright (c) 2012-2013 Synergetics SA (sampo@synergetics.be), All Rights Reserved.
+ * Copyright (c) 2008-2011 Sampo Kellomaki (sampo@iki.fi), All Rights Reserved.
  * This is confidential unpublished proprietary source code of the author.
  * NO WARRANTY, not even implied warranties. Contains trade secrets.
  * Distribution prohibited unless authorized in writing.
@@ -8,6 +9,7 @@
  *
  * 12.11.2008, created --Sampo
  * 24.8.2009,  perfected for TAS3 workshop --Sampo
+ * 13.12.2011, added  VPATH and VURL --Sampo
  *
  * See zxid_idp_dispatch() in zxididpx.c for most interesting parts of IdP implementation.
  *
@@ -32,6 +34,7 @@
 char* help =
 "zxididp  -  SAML 2.0 IdP CGI (also DI, IM, and PS) - R" ZXID_REL "\n\
 SAML 2.0 is a standard for federated identity and Single Sign-On.\n\
+Copyright (c) 2012-2013 Synergetics NV (sampo@synergetics.be), All Rights Reserved.\n\
 Copyright (c) 2008-2011 Sampo Kellomaki (sampo@iki.fi), All Rights Reserved.\n\
 NO WARRANTY, not even implied warranties. Licensed under Apache License v2.0\n\
 See http://www.apache.org/licenses/LICENSE-2.0\n\
@@ -47,9 +50,12 @@ Usage: zxididp [options]   (when used as CGI, no options can be supplied)\n\
 /* CONFIG: You must edit the URL to match your domain name and port */
 
 #ifdef MINGW
-#define CONF "URL=https://idp1.zxidp.org:8443/zxididp&NICE_NAME=ZXIdP&NOSIG_FATAL=0&SES_COOKIE_NAME=ZXIDPSES&IDP_ENA=1&PDP_ENA=1&PATH=c:/var/zxid/idp"
+#define CONF "URL=https://idp1.zxidp.org:8443/zxididp&SES_COOKIE_NAME=ZXIDPSES&IDP_ENA=1&PDP_ENA=1&PATH=c:/var/zxid/idp"
 #else
-#define CONF "URL=https://idp1.zxidp.org:8443/zxididp&NICE_NAME=ZXIdP&NOSIG_FATAL=0&SES_COOKIE_NAME=ZXIDPSES&IDP_ENA=1&PDP_ENA=1&PATH=/var/zxid/idp"
+/*#define CONF "URL=https://idp1.zxidp.org:8443/zxididp&NICE_NAME=ZXIdP&NOSIG_FATAL=0&SES_COOKIE_NAME=ZXIDPSES&IDP_ENA=1&PDP_ENA=1&PATH=/var/zxid/idp"*/
+//#define CONF "IDP_ENA=1&VPATH=%h/&VURL=%a%h%s"
+//#define CONF "IDP_ENA=1&PATH=/var/zxid/idp&VPATH=/var/zxid/%h/&VURL=%a%h%s"
+#define CONF "IDP_ENA=1"
 #endif
 
 /* Called by: */
@@ -61,16 +67,36 @@ int main(int argc, char** argv)
   char* res;
   char* setcookie;
 
+#ifdef _GNU_SOURCE
+  if (getenv("MALLOC_TRACE"))
+    mtrace();
+#endif
+
+#if 0
+  /* Allocate and realase memory to cause malloc to grab bigger mmap page */
+  /* Apparently this trick does not work - perhaps memory allocation
+     is sorted by page size or something. --Sampo */
+#ifndef ZXIDIDP_PREALLOC_KB
+#define ZXIDIDP_PREALLOC_KB 300
+#endif
+  free(malloc(ZXIDIDP_PREALLOC_KB*1024));
+#endif
+
 #if 1
   /* Helps debugging CGI scripts if you see stderr. */
   /* Reopen stderr only in mini_httpd case */
   p = getenv("SERVER_SOFTWARE");
   if (p && !memcmp(p, "mini_httpd", sizeof("mini_httpd")-1)) {
     close(2);
-    if (open("/var/tmp/zxid.stderr", O_WRONLY | O_CREAT | O_APPEND, 0666) != 2)
+    if (open("/var/tmp/zxid.stderr", O_WRONLY | O_CREAT | O_APPEND, 0666) != 2) {
+      perror("/var/tmp/zxid.stderr");
       exit(2);
+    }
   }
-  fprintf(stderr, "=================== Running zxididp %s ===================\n", ZXID_REL);
+  /*zx_debug = 1;*/
+  fprintf(stderr, "=================== Running zxididp %s =================== %d\n", ZXID_REL, zx_debug);
+  p = getenv("ZXID_PRE_CONF");
+  D("ZXID_PRE_CONF(%s)", p);
   //fprintf(stderr, "p(%s)\n", p);
 #endif
 
@@ -80,12 +106,12 @@ int main(int argc, char** argv)
   }
 
 #if 0
-  strncpy(zx_instance, "\t\e[47mzxidp\e[0m", sizeof(zx_instance));
+  strncpy(zx_instance, "\e[47midp\e[0m", sizeof(zx_instance));
 #else
-  strncpy(zx_instance, "\tzxidp", sizeof(zx_instance));
+  strncpy(zx_instance, "idp", sizeof(zx_instance));
 #endif
-  //zx_debug = 1;
-  res = zxid_simple(CONF, 0, 0x1fff);  /* 0xfff == full CGI automation */
+
+  res = zxid_simple(CONF, 0, 0x0fff);  /* 0xfff == full CGI automation */
   switch (res[0]) {
   default:
     ERR("Unknown zxid_simple() response(%s)", res);
