@@ -138,7 +138,7 @@ static struct zx_str* zxid_pool_to_ldif(zxid_conf* cf, struct zxid_attr* pool)
 	p += av->map_val->len;
 	*p++ = '\n';
 
-	DD("len 2=%d", ((int)(p-ss->s)));
+	DD("len 2=%d", (int)(p-ss->s));
       }
 
 
@@ -154,7 +154,7 @@ static struct zx_str* zxid_pool_to_ldif(zxid_conf* cf, struct zxid_attr* pool)
       }
       *p++ = '\n';
 
-      DD("len 3=%d name_len=%d name(%s)", ((int)(p-ss->s)), name_len, at->name);
+      DD("len 3=%d name_len=%d name(%s)", (int)(p-ss->s), name_len, at->name);
       
       for (av = at->nv; av; av = av->n) {
 	strcpy(p, at->name);
@@ -167,19 +167,89 @@ static struct zx_str* zxid_pool_to_ldif(zxid_conf* cf, struct zxid_attr* pool)
 	}
 	*p++ = '\n';
 
-	D("len 4=%d", ((int)(p-ss->s)));
+	D("len 4=%d", (int)(p-ss->s));
       }
 
     }
   }
-  DD("len Fin=%d", ((int)(p-ss->s)));
+  DD("len Fin=%d", (int)(p-ss->s));
 
-  ASSERTOP(p, ==, ss->s+len);
+  ASSERTOPP(p, ==, ss->s+len);
   return ss;
 }
 
-/*() Convert attributes from (session) pool to JSON, applying OUTMAP.
- * *** Need to check escaping JSON values, e.g. " or \n */
+/*(-) Length computation of JSON string */
+
+/* Called by:  zxid_pool_to_json x9 */
+static int zxid_json_strlen(char* js)
+{
+  int res = 0;
+  for (; *js; ++js, ++res) {
+    int c = *(unsigned char*)js;
+    if (c < ' ') {
+      if ((c == '\n') || (c == '\r') || (c == '\t') ||
+	  (c == '\b') || (c == '\f')) {
+	/* \X */
+	res++;
+      } else {
+	/* \uXXXX */
+	res += 5;
+      }
+    } else if ((c == '\'') || (c == '\"') || (c == '\\')) {
+      /* \X */
+      res++;
+    } else if ((c == 0xe2) && (((unsigned char*)js)[1] == 0x80) &&
+	       ((((unsigned char*)js)[2] & 0xfe) == 0xa8)) {
+      /* Some java-script based JSON decoders don't like
+       * unescaped \u2028 and \u2029. */
+      /* \uXXXX */
+      res += 5;
+      js += 2;
+    }
+  }
+  return res;
+}
+
+/*(-) Copy JSON string */
+
+/* Called by:  zxid_pool_to_json x8 */
+static char* zxid_json_strcpy(char* dest, char* js)
+{
+  for (; *js; ++js) {
+    int c = *(unsigned char*)js;
+    if (c < ' ') {
+      /* Control character. */
+      *dest++ = '\\';
+      if (c == '\n') c = 'n';
+      else if (c == '\r') c = 'r';
+      else if (c == '\t') c = 't';
+      else if (c == '\b') c = 'b';
+      else if (c == '\f') c = 'f';
+      else {
+	/* \uXXXX */
+	sprintf(dest, "u%04x", c);
+	dest += 5;
+	continue;
+      }
+    } else if ((c == '\'') || (c == '\"') || (c == '\\')) {
+      /* \X */
+      *dest++ = '\\';
+    } else if ((c == 0xe2) && (((unsigned char*)js)[1] == 0x80) &&
+	       ((((unsigned char*)js)[2] & 0xfe) == 0xa8)) {
+      /* Some java-script based JSON decoders don't like
+       * unescaped \u2028 and \u2029. */
+      /* \uXXXX */
+      sprintf(dest, "\\u%04x", 0x2028 | (js[2] & 1));
+      js += 2;
+      dest += 6;
+      continue;
+    }
+    *dest++ = c;
+  }
+  return dest;
+}
+
+/*() Convert attributes from (session) pool to JSON, applying OUTMAP. */
 
 /* Called by:  zxid_ses_to_json */
 static struct zx_str* zxid_pool_to_json(zxid_conf* cf, struct zxid_attr* pool)
@@ -203,28 +273,34 @@ static struct zx_str* zxid_pool_to_json(zxid_conf* cf, struct zxid_attr* pool)
       }
       at->map_val = zxid_map_val(cf, 0, 0, map, at->name, at->val);
       if (map->dst && *map->dst && map->src && map->src[0] != '*') {
-	name_len = strlen(map->dst);
+	name_len = zxid_json_strlen(map->dst);
       } else {
-	name_len = strlen(at->name);
+	name_len = zxid_json_strlen(at->name);
       }
 
       if (at->nv) {  /* Multivalue requires array */
-	len += name_len + sizeof("\"\":[\"\"],")-1 + at->map_val->len;
+	len += name_len + sizeof("\"\":[\"\"],")-1 +
+	  zxid_json_strlen(at->map_val->s);
 	for (av = at->nv; av; av = av->n) {
 	  av->map_val = zxid_map_val(cf, 0, 0, map, at->name, av->val);
-	  len += name_len + sizeof(",\"\"")-1 + av->map_val->len;
+	  len += name_len + sizeof(",\"\"")-1 +
+	    zxid_json_strlen(at->map_val->s);
 	}
       } else {
-	len += name_len + sizeof("\"\":\"\",")-1 + at->map_val->len;
+	len += name_len + sizeof("\"\":\"\",")-1 +
+	  zxid_json_strlen(at->map_val->s);
       }
     } else {
-      name_len = strlen(at->name);
+      name_len = zxid_json_strlen(at->name);
       if (at->nv) {  /* Multivalue requires array */
-	len += name_len + sizeof("\"\":[\"\"],")-1 + (at->val?strlen(at->val):0);
+	len += name_len + sizeof("\"\":[\"\"],")-1 +
+	  (at->val?zxid_json_strlen(at->val):0);
 	for (av = at->nv; av; av = av->n)
-	  len += name_len + sizeof(",\"\"")-1 + (av->val?strlen(av->val):0);
+	  len += name_len + sizeof(",\"\"")-1 +
+	    (av->val?zxid_json_strlen(av->val):0);
       } else {
-	len += name_len + sizeof("\"\":\"\",")-1 + (at->val?strlen(at->val):0);
+	len += name_len + sizeof("\"\":\"\",")-1 +
+	  (at->val?zxid_json_strlen(at->val):0);
       }
     }
   }
@@ -247,51 +323,45 @@ static struct zx_str* zxid_pool_to_json(zxid_conf* cf, struct zxid_attr* pool)
       }
 
       *p++ = '"';
-      strcpy(p, name);
+      p = zxid_json_strcpy(p, name);
       p += strlen(name);
       *p++ = '"';
       *p++ = ':';
       if (at->nv) {
 	*p++ = '[';
 	*p++ = '"';
-	memcpy(p, at->map_val->s, at->map_val->len);
-	p += at->map_val->len;
+	p = zxid_json_strcpy(p, at->map_val->s);
 	*p++ = '"';
 	for (av = at->nv; av; av = av->n) {
 	  *p++ = ',';
 	  *p++ = '"';
-	  memcpy(p, av->map_val->s, av->map_val->len);
-	  p += av->map_val->len;
+	  p = zxid_json_strcpy(p, av->map_val->s);
 	  *p++ = '"';
 	}
 	*p++ = ']';
       } else {
 	*p++ = '"';
-	memcpy(p, at->map_val->s, at->map_val->len);
-	p += at->map_val->len;
+	p = zxid_json_strcpy(p, at->map_val->s);
 	*p++ = '"';
       }
 
     } else {
       *p++ = '"';
-      strcpy(p, at->name);
-      p += strlen(at->name);
+      p = zxid_json_strcpy(p, at->name);
       *p++ = '"';
       *p++ = ':';
       if (at->nv) {
 	*p++ = '[';
 	*p++ = '"';
 	if (at->val) {
-	  strcpy(p, at->val);
-	  p += strlen(at->val);
+	  p = zxid_json_strcpy(p, at->val);
 	}
 	*p++ = '"';
 	for (av = at->nv; av; av = av->n) {
 	  *p++ = ',';
 	  *p++ = '"';
 	  if (at->val) {
-	    strcpy(p, av->val);
-	    p += strlen(av->val);
+	    p = zxid_json_strcpy(p, av->val);
 	  }
 	  *p++ = '"';
 	}
@@ -299,16 +369,15 @@ static struct zx_str* zxid_pool_to_json(zxid_conf* cf, struct zxid_attr* pool)
       } else {
 	*p++ = '"';
 	if (at->val) {
-	  strcpy(p, at->val);
-	  p += strlen(at->val);
+	  p = zxid_json_strcpy(p, at->val);
 	}
 	*p++ = '"';
       }
     }
     *p++ = ',';
   }
-  *p = '}';   /* Overwrites last comma */
-  ASSERTOP(p, ==, ss->s+len);
+  p[-1] = '}';   /* Overwrites last comma */
+  ASSERTOPP(p, ==, ss->s+len);
   return ss;
 }
 
@@ -413,8 +482,8 @@ static struct zx_str* zxid_pool_to_qs(zxid_conf* cf, struct zxid_attr* pool)
     }
   }
   D("p=%p == %p ss=%p len=%d", p, ss->s+len, ss->s, len);
-  D("p(%.*s)", len, ss->s);
-  ASSERTOP(p, ==, ss->s+len);
+  DD("p(%.*s)", len, ss->s);
+  ASSERTOPP(p, ==, ss->s+len);
   *p = 0;  /* Zap last & */
   return ss;
 }
@@ -440,10 +509,10 @@ struct zx_str* zxid_ses_to_qs(zxid_conf* cf, zxid_ses* ses) {
   return zxid_pool_to_qs(cf, ses?ses->at:0);
 }
 
-/*() Add values, applying NEED, WANT, and INMAP */
+/*() Add values to session attribute pool, applying NEED, WANT, and INMAP */
 
 /* Called by:  zxid_add_a7n_at_to_pool x2 */
-static int zxid_add_at_values(zxid_conf* cf, zxid_ses* ses, struct zx_sa_Attribute_s* at, char* name, struct zx_str* issuer)
+static int zxid_add_at_vals(zxid_conf* cf, zxid_ses* ses, struct zx_sa_Attribute_s* at, char* name, struct zx_str* issuer)
 {
   struct zx_str* ss;
   struct zxid_map* map;
@@ -463,7 +532,7 @@ static int zxid_add_at_values(zxid_conf* cf, zxid_ses* ses, struct zx_sa_Attribu
   }
   
   if (map && map->dst && *map->dst && map->src && map->src[0] != '*') {
-    ses->at = zxid_new_at(cf, ses->at, strlen(map->dst), map->dst, 0, 0, "mapped");
+    ses->at = zxid_new_at(cf, ses->at, strlen(map->dst), map->dst, 0, 0, "mappd");
   } else {
     ses->at = zxid_new_at(cf, ses->at, strlen(name), name, 0, 0, "as is");
   }
@@ -475,16 +544,16 @@ static int zxid_add_at_values(zxid_conf* cf, zxid_ses* ses, struct zx_sa_Attribu
        av = (struct zx_sa_AttributeValue_s*)ZX_NEXT(av)) {
     if (av->gg.g.tok != zx_sa_AttributeValue_ELEM)
       continue;
-    D("Adding value: %p", ZX_GET_CONTENT(av));
+    DD("  adding value: %p", ZX_GET_CONTENT(av));
     if (av->EndpointReference || av->ResourceOffering)
       continue;  /* Skip bootstraps. They are handled elsewhere, see zxid_snarf_eprs_from_ses(). */
     if (ZX_GET_CONTENT(av)) {
       ss = zxid_map_val_ss(cf, ses, 0, map, ses->at->name, ZX_GET_CONTENT(av));
       if (ses->at->val) {
-	D("map val(%.*s)", ss->len, ss->s);
+	D("  map val(%.*s)", ss->len, ss->s);
 	ses->at->nv = zxid_new_at(cf, ses->at->nv, 0, 0, ss->len, ss->s, "multival");
       } else {
-	D("copy val(%.*s)", ss->len, ss->s);
+	D("  copy val(%.*s)", ss->len, ss->s);
 	COPYVAL(ses->at->val, ss->s, ss->s+ss->len);
       }
     }
@@ -493,7 +562,7 @@ static int zxid_add_at_values(zxid_conf* cf, zxid_ses* ses, struct zx_sa_Attribu
   return 1;
 }
 
-/*() Add Attribute Statements of an Assertion to pool, applying NEED, WANT, and INMAP */
+/*() Add Attribute Statements of an Assertion to session attribute pool, applying NEED, WANT, and INMAP */
 
 /* Called by:  zxid_ses_to_pool */
 static void zxid_add_a7n_at_to_pool(zxid_conf* cf, zxid_ses* ses, zxid_a7n* a7n)
@@ -514,16 +583,17 @@ static void zxid_add_a7n_at_to_pool(zxid_conf* cf, zxid_ses* ses, zxid_a7n* a7n)
       if (at->gg.g.tok != zx_sa_Attribute_ELEM)
 	continue;
       if (at->Name)
-	zxid_add_at_values(cf, ses, at, zx_str_to_c(cf->ctx, &at->Name->g), ZX_GET_CONTENT(a7n->Issuer));
+	zxid_add_at_vals(cf, ses, at, zx_str_to_c(cf->ctx, &at->Name->g), ZX_GET_CONTENT(a7n->Issuer));
       if (at->FriendlyName)
-	zxid_add_at_values(cf, ses, at, zx_str_to_c(cf->ctx, &at->FriendlyName->g), ZX_GET_CONTENT(a7n->Issuer));
+	zxid_add_at_vals(cf, ses, at, zx_str_to_c(cf->ctx, &at->FriendlyName->g), ZX_GET_CONTENT(a7n->Issuer));
     }
   }
 }
 
-/*() Add simple attribute to pool, applying NEED, WANT, and INMAP */
+/*() Add simple attribute to session's attribute pool, applying NEED, WANT, and INMAP.
+ * Replaces zxid_add_attr_to_pool() */
 
-/* Called by:  zxid_add_ldif_at2ses, zxid_add_qs_to_ses, zxid_ses_to_pool x25 */
+/* Called by:  chkuid, zxid_add_ldif_at2ses, zxid_add_qs2ses, zxid_ses_to_pool x25 */
 void zxid_add_attr_to_ses(zxid_conf* cf, zxid_ses* ses, char* at_name, struct zx_str* val)
 {
   struct zxid_map* map;
@@ -536,7 +606,7 @@ void zxid_add_attr_to_ses(zxid_conf* cf, zxid_ses* ses, char* at_name, struct zx
       D("attribute(%s) filtered out by del rule in INMAP", at_name);
     } else {
       if (map && map->dst && *map->dst && map->src && map->src[0] != '*') {
-	ses->at = zxid_new_at(cf, ses->at, strlen(map->dst), map->dst, val->len, val->s, "mapd2");
+	ses->at = zxid_new_at(cf, ses->at, strlen(map->dst), map->dst, val->len, val->s, "mappd2");
       } else {
 	ses->at = zxid_new_at(cf, ses->at, strlen(at_name), at_name, val->len, val->s, "as is2");
       }
@@ -575,7 +645,7 @@ static void zxid_add_ldif_at2ses(zxid_conf* cf, zxid_ses* ses, const char* prefi
     val = p+2;
     p = strchr(val, '\n');  /* *** parsing LDIF is fragile if values are multiline */
     len = p?(p-val):strlen(val);
-    D("%s: ATTR(%s)=VAL(%.*s)", lk, name_buf, len, val);
+    D("%s: ATTR(%s)=(%.*s)", lk, name_buf, len, val);
     zxid_add_attr_to_ses(cf, ses, name_buf,  zx_dup_len_str(cf->ctx, len, val));
   }
 }
@@ -626,8 +696,8 @@ static void zxid_cp_usr_eprs2ses(zxid_conf* cf, zxid_ses* ses, struct zx_str* pa
   closedir(dir);
 }
 
-/*(i) Process attributes from the AttributeStatements of the session
- * SSO Assertion and insert them to the pool. NEED, WANT, and INMAP
+/*(i) Process attributes from the AttributeStatements of the session's
+ * SSO Assertion and insert them to the session's attribute pool. NEED, WANT, and INMAP
  * are applied. The pool is suitable for use by PEP or eventually
  * rendering to LDIF (or JSON). This function also implements
  * local attribute authority. */
@@ -739,6 +809,7 @@ void zxid_ses_to_pool(zxid_conf* cf, zxid_ses* ses)
   }
   zxid_add_attr_to_ses(cf, ses, "sesix",      zx_dup_str(cf->ctx, STRNULLCHK(ses->sesix)));
   zxid_add_attr_to_ses(cf, ses, "setcookie",  zx_dup_str(cf->ctx, STRNULLCHK(ses->setcookie)));
+  zxid_add_attr_to_ses(cf, ses, "setptmcookie",zx_dup_str(cf->ctx,STRNULLCHK(ses->setptmcookie)));
   if (ses->cookie && ses->cookie[0])
     zxid_add_attr_to_ses(cf, ses, "cookie",   zx_dup_str(cf->ctx, ses->cookie));
   zxid_add_attr_to_ses(cf, ses, "msgid",      ses->wsp_msgid);
@@ -753,7 +824,8 @@ void zxid_ses_to_pool(zxid_conf* cf, zxid_ses* ses)
 }
 
 /*(i) Add Attributes from Querty String to Session attribute pool
- * The qs argument is parsed according to the CGI Query String rules
+ * The qs argument is parsed according to the CGI Query String rules (string
+ * is modifed to insert nul terminations and URL decoded in place)
  * and the attributes are added to the session. If apply_map is 1, the
  * INMAP configuration is applied. While this may seem a hassle, it
  * allows for specification of the values as safe_base64, etc. If values
@@ -763,42 +835,25 @@ void zxid_ses_to_pool(zxid_conf* cf, zxid_ses* ses)
  * Returns 1 on success, 0 on failure (return value often not checked). */
 
 /* Called by:  zxid_az_base_cf_ses, zxid_az_cf_ses */
-int zxid_add_qs_to_ses(zxid_conf* cf, zxid_ses* ses, char* qs, int apply_map)
+int zxid_add_qs2ses(zxid_conf* cf, zxid_ses* ses, char* qs, int apply_map)
 {
-  char *p, *n, *v, *val, *name;
+  char* n;
+  char* v;
   if (!qs || !ses)
     return 0;
 
   D("qs(%s) len=%d", qs, (int)strlen(qs));
   while (qs && *qs) {
-    for (; *qs == '&'; ++qs) ;    /* Skip over & or && */
-    if (!*qs) break;
-    
-    qs = strchr(name = qs, '=');  /* Scan name (until '=') */
-    if (!qs) break;
-    if (qs == name) {             /* Key was an empty string: skip it */
-      qs = strchr(qs, '&');       /* Scan value (until '&') *** or '?' */
-      continue;
-    }
-    for (; name < qs && *name <= ' '; ++name) ; /* Skip over initial whitespace before name */
-    n = p = name;
-    URL_DECODE(p, name, qs);
-    *p = 0;                                     /* Nul-term n (name) */
-    
-    for (val = ++qs; *qs && *qs != '&'; ++qs) ; /* Skip over = and scan value till '&' */
-    v = p = val;
-    URL_DECODE(p, val, qs);
-
-    if (*qs)
-      ++qs;
-    *p = 0;                                     /* Nul-term v (value) */
+    qs = zxid_qs_nv_scan(qs, &n, &v, 1);
+    if (!n)
+      n = "NULL_NAM_ERR";
 
     if (apply_map) {
       D("map %s=%s", n,v);
       zxid_add_attr_to_ses(cf, ses, n, zx_dup_str(cf->ctx, v));  
     } else {
       D("asis %s=%s", n,v);
-      ses->at = zxid_new_at(cf, ses->at, v-n-1, n, p-v, v, "as is");
+      ses->at = zxid_new_at(cf, ses->at, strlen(n), n, strlen(v), v, "as is3");
     }
   }
   return 1;
